package core.services;

import core.api.models.timemanager.taskSchedule.scheduleInfos.AdvancedScheduleFieldInfo;
import core.api.models.timemanager.taskSchedule.scheduleInfos.AdvancedScheduleInfo;
import core.api.models.timemanager.taskSchedule.scheduleInfos.BasicScheduleFieldInfo;
import core.api.models.timemanager.taskSchedule.ForgottenScheduleInfo;
import core.entities.timemanager.AbstractSchedule;
import core.entities.timemanager.AdvancedTaskSchedule;
import core.entities.timemanager.BasicTaskSchedule;
import core.entities.timemanager.Task;
import core.repositories.UserRepository;
import core.repositories.timemanager.ScheduleRepository;
import core.repositories.timemanager.TaskRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Optional;

@Service
public class TaskScheduleService {

    @Autowired private ScheduleRepository scheduleRepository;
    @Autowired private UserRepository userRepository;
    @Autowired
    private TaskRepository taskRepository;

    public List<AbstractSchedule> getAllSchedulesOfUser(String username) {
        return scheduleRepository.findAllByUsername(username);
    }

    public ServiceResult<AbstractSchedule> scheduleBasic(Task task, BasicScheduleFieldInfo scheduleFieldInfo) {
        if(task.isFinished() || scheduleFieldInfo.getScheduleDate().isBefore(LocalDate.now())) {
            return new ServiceResult<>(ServiceExitCode.INVALID_OPERATION);
        }

        BasicTaskSchedule basicTaskSchedule = new BasicTaskSchedule(task, scheduleFieldInfo.getScheduleDate());
        scheduleRepository.save(basicTaskSchedule);

        return new ServiceResult<>(basicTaskSchedule);
    }

    public ServiceResult<AbstractSchedule> scheduleAdvanced(Task task, AdvancedScheduleFieldInfo scheduleFieldInfo) {
        LocalDate startDate = scheduleFieldInfo.getScheduleStartTime().toLocalDate();
        LocalDate endDate = scheduleFieldInfo.getScheduleStopTime().toLocalDate();

        if(task.isFinished() || startDate.isBefore(LocalDate.now()) || endDate.isBefore(LocalDate.now())) {
            return new ServiceResult<>(ServiceExitCode.INVALID_OPERATION);
        }

        if(scheduleFieldInfo.getScheduleStartTime().isAfter(scheduleFieldInfo.getScheduleStopTime())) {
            return new ServiceResult<>(ServiceExitCode.INVALID_PARAMETER);
        }

        AdvancedTaskSchedule advancedTaskSchedule = new AdvancedTaskSchedule(task, scheduleFieldInfo.getScheduleStartTime(), scheduleFieldInfo.getScheduleStopTime());
        scheduleRepository.save(advancedTaskSchedule);

        return new ServiceResult<>(advancedTaskSchedule);
    }

    public PermissionResult<AbstractSchedule> getSchedulePermissions(long scheduleID, String username) {
        Optional<AbstractSchedule> abstractSchedule = scheduleRepository.findById(scheduleID);
        return abstractSchedule.map(schedule -> new PermissionResult<>(schedule, schedule.getTask().getTaskgroup().getUser().getUsername().equals(username))).orElseGet(() -> new PermissionResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public ServiceResult<AbstractSchedule> editBasicSchedule(BasicTaskSchedule schedule, BasicScheduleFieldInfo scheduleFieldInfo) {
        if(schedule.getTask().isFinished() || scheduleFieldInfo.getScheduleDate().isBefore(LocalDate.now()) || schedule.isActive()) {
            return new ServiceResult<>(ServiceExitCode.INVALID_OPERATION);
        }

        schedule.setScheduleDate(scheduleFieldInfo.getScheduleDate());
        scheduleRepository.save(schedule);
        return new ServiceResult<>(schedule);
    }

    public ServiceResult<AbstractSchedule> editAdvancedSchedule(AdvancedTaskSchedule schedule, AdvancedScheduleFieldInfo scheduleFieldInfo) {
        LocalDate startDate = scheduleFieldInfo.getScheduleStartTime().toLocalDate();
        LocalDate endDate = scheduleFieldInfo.getScheduleStopTime().toLocalDate();

        if(schedule.getTask().isFinished() || startDate.isBefore(LocalDate.now()) || endDate.isBefore(LocalDate.now()) || schedule.isActive()) {
            return new ServiceResult<>(ServiceExitCode.INVALID_OPERATION);
        }

        if(scheduleFieldInfo.getScheduleStartTime().isAfter(scheduleFieldInfo.getScheduleStopTime())) {
            return new ServiceResult<>(ServiceExitCode.INVALID_PARAMETER);
        }

        schedule.setScheduleStart(scheduleFieldInfo.getScheduleStartTime());
        schedule.setScheduleEnd(scheduleFieldInfo.getScheduleStopTime());
        scheduleRepository.save(schedule);
        return new ServiceResult<>(schedule);
    }

    public void deleteSchedule(AbstractSchedule schedule) {
        scheduleRepository.delete(schedule);
    }

    public List<AbstractSchedule> getFilteredScheduledOfUser(LocalDate date, boolean startable, String name) {
        List<AbstractSchedule> abstractSchedules = getAllSchedulesOfUser(name);
        List<AbstractSchedule> filteredSchedules = new LinkedList<>();
        for(AbstractSchedule abstractSchedule : abstractSchedules) {
            if(abstractSchedule instanceof BasicTaskSchedule) {
                if(((BasicTaskSchedule) abstractSchedule).getScheduleDate().isEqual(date)) {
                    //Schedule is today
                    if(startable && abstractSchedule.getStartTime() == null) {
                        if(abstractSchedule.getStartTime() == null) {
                            filteredSchedules.add(abstractSchedule);
                        }
                    } else if(!startable) {
                        filteredSchedules.add(abstractSchedule);
                    }
                }
            } else {
                if(((AdvancedTaskSchedule) abstractSchedule).getScheduleStart().toLocalDate().isEqual(LocalDate.now())) {
                    //Schedule is today
                    if(startable && abstractSchedule.getStartTime() == null) {
                        if(abstractSchedule.getStartTime() == null) {
                            filteredSchedules.add(abstractSchedule);
                        }
                    } else if(!startable) {
                        filteredSchedules.add(abstractSchedule);
                    }
                }
            }
        }
        return filteredSchedules;
    }

    public ServiceResult<AbstractSchedule> scheduleNow(Task task) {
        if(task.isFinished()) {
            return new ServiceResult<>(ServiceExitCode.INVALID_OPERATION);
        }

        if(task.hasActiveSchedule()) {
            return new ServiceResult<>(ServiceExitCode.ENTITY_ALREADY_EXIST);
        } else {
            BasicTaskSchedule basicTaskSchedule = new BasicTaskSchedule(task, LocalDateTime.now());
            scheduleRepository.save(basicTaskSchedule);
            return new ServiceResult<>(basicTaskSchedule);
        }
    }

    public ServiceResult<AbstractSchedule> getActiveSchedule(String username) {
        Optional<AbstractSchedule> activeSchedule = scheduleRepository.getActiveScheduleOfUser(username);
        return activeSchedule.map(ServiceResult::new).orElseGet(() -> new ServiceResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public ServiceResult<AbstractSchedule> activateSchedule(AbstractSchedule schedule) {
        if(schedule.getStartTime() == null) {
            schedule.setStartTime(LocalDateTime.now());
            scheduleRepository.save(schedule);
        }
        return new ServiceResult<>(schedule);
    }

    public ServiceResult<AbstractSchedule> stopSchedule(AbstractSchedule schedule, boolean finish) {
        if(schedule.getStartTime() != null && schedule.getStopTime() == null) {
            schedule.setStopTime(LocalDateTime.now());
            scheduleRepository.save(schedule);
        }

        if(finish) {
            schedule.getTask().setFinished(true);
            taskRepository.save(schedule.getTask());

            List<AbstractSchedule> startableSchedules = schedule.getTask().getStartableSchedules();
            scheduleRepository.deleteAll(startableSchedules);
        }
        return new ServiceResult<>(schedule);
    }

    public ServiceResult<AbstractSchedule> registerForgottenSchedule(Task task, ForgottenScheduleInfo forgottenScheduleInfo) {
        if(task.isFinished()) {
            return new ServiceResult<>(ServiceExitCode.INVALID_OPERATION);
        }

        BasicTaskSchedule basicTaskSchedule = new BasicTaskSchedule(task, forgottenScheduleInfo.getStartTime(), forgottenScheduleInfo.getEndTime(), forgottenScheduleInfo.getStartTime().toLocalDate());
        scheduleRepository.save(basicTaskSchedule);

        return new ServiceResult<>(basicTaskSchedule);
    }

    public List<AbstractSchedule> getAllMissedSchedulesOfUser(String username) {
        List<AbstractSchedule> missedSchedules = new ArrayList<>();
        List<AbstractSchedule> schedules = getAllSchedulesOfUser(username);
        LocalDateTime now = LocalDateTime.now();
        for(AbstractSchedule schedule : schedules) {
            if(schedule.isMissed(now)) {
                missedSchedules.add(schedule);
            }
        }
        return missedSchedules;
    }

    public List<AbstractSchedule> getActiveSchedulesOfToday(String username) {
        List<AbstractSchedule> activatedSchedules = new ArrayList<>();
        for(AbstractSchedule schedule : getAllSchedulesOfUser(username)) {
            if(schedule.getStopTime() != null && schedule.getStopTime().toLocalDate().isEqual(LocalDate.now())) {
                activatedSchedules.add(schedule);
            }
        }
        return activatedSchedules;
    }
}
