package core.taskgroups;

import core.entities.User;
import core.entities.timemanager.Task;
import core.entities.timemanager.Taskgroup;
import core.repositories.timemanager.TaskgroupRepository;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.test.context.jdbc.Sql;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.util.List;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.junit.jupiter.api.Assertions.*;

@ExtendWith(SpringExtension.class)
@DataJpaTest
public class TaskgroupRepsitoryTest {

    @Autowired private TaskgroupRepository taskgroupRepository;
    @Autowired private TestEntityManager testEntityManager;

    @Test
    @Sql("classpath:taskgroupRepositoryTestEntries.sql")
    void findAllByUser() {
        User testUser1 = testEntityManager.find(User.class, 1L);
        User testUser2 = testEntityManager.find(User.class, 2L);

        List<Taskgroup> result_user2 = taskgroupRepository.findAllByUser(testUser2.getUsername());
        assertEquals(0, result_user2.size());

        List<Taskgroup> result_user1 = taskgroupRepository.findAllByUser(testUser1.getUsername());
        assertEquals(8, result_user1.size());
    }

    @Test
    @Sql("classpath:taskgroupRepositoryTestEntries.sql")
    void existsByTaskgroupNameAndUser() {
        User testUser1 = testEntityManager.find(User.class, 1L);
        User testUser2 = testEntityManager.find(User.class, 2L);

        //Situation 1: Taskgroup exists but within another user
        assertFalse(taskgroupRepository.existsByTaskgroupNameAndUser("No children", testUser2));

        //Situation 2: Taskgroup exists not
        assertFalse(taskgroupRepository.existsByTaskgroupNameAndUser("ada", testUser1));

        //Situation 3: Taskgroup (top) exists on user
        assertTrue(taskgroupRepository.existsByTaskgroupNameAndUser("No children", testUser1));

        //Situation 4: Taskgroup exists on user but not on top level
        assertTrue(taskgroupRepository.existsByTaskgroupNameAndUser("Taskgroup 1.1", testUser1));
    }

    @Test
    @Sql("classpath:taskgroupRepositoryTestEntries.sql")
    void findAllTopTaskgroupsByUser() {
        User testUser1 = testEntityManager.find(User.class, 1L);
        User testUser2 = testEntityManager.find(User.class, 2L);

        //Situation 1: Empty user, no taskgroups
        assertEquals(0, taskgroupRepository.findAllTopTaskgroupsByUser(testUser2.getUsername()).size());

        //Situation 2: Only top taskgroups are returned
        List<Taskgroup> topgroups_user1 = taskgroupRepository.findAllTopTaskgroupsByUser(testUser1.getUsername());
        assertEquals(3, topgroups_user1.size());
        assertTrue(topgroups_user1.contains(testEntityManager.find(Taskgroup.class, 1L)));
        assertTrue(topgroups_user1.contains(testEntityManager.find(Taskgroup.class, 2L)));
        assertTrue(topgroups_user1.contains(testEntityManager.find(Taskgroup.class, 5L)));

        //Situation 3: User with username does not exist
        assertEquals(0, taskgroupRepository.findAllTopTaskgroupsByUser("Rotzbakke").size());
    }
    @Test
    @Sql("classpath:taskgroupRepositoryTestEntries.sql")
    void deleteAllByUser() {
        User testUser1 = testEntityManager.find(User.class, 1L);
        User testUser2 = testEntityManager.find(User.class, 2L);

        taskgroupRepository.deleteAllByUser(testUser2);
        for(long i=1; i<=8; i++) {
            assertThat(testEntityManager.find(Taskgroup.class, i)).isNotNull();
        }

        taskgroupRepository.deleteAllByUser(testUser1);
        for(long i=1; i<=8; i++) {
            assertThat(testEntityManager.find(Taskgroup.class, i)).isNull();
        }
    }

    @Test
    @Sql("classpath:taskgroupRepositoryTestEntries.sql")
    void delete() {
        Taskgroup no_children = testEntityManager.find(Taskgroup.class, 1L);
        Taskgroup taskgroup_with_children = testEntityManager.find(Taskgroup.class, 5L);

        taskgroupRepository.delete(no_children);
        assertThat(testEntityManager.find(Taskgroup.class, 1L)).isNull();
        for(long i=2; i<=8; i++) {
            assertThat(testEntityManager.find(Taskgroup.class, i)).isNotNull();
        }

        taskgroupRepository.delete(taskgroup_with_children);
        for(long i=5; i<=8; i++) {
            assertThat(testEntityManager.find(Taskgroup.class, i)).isNull();
        }
    }
}
